@@ -58,9 +58,9 @@ plot(ecmla)
 # Figure 9.5 (a-b)
 # Using Empirical Margins For Figure 9.5(a)
 # Using Gev Margins For Figure 9.5(b)
-abvnonpar(data = cmla, epmar = TRUE, method = "p", rev = TRUE, plot = TRUE, lty = 3)
-abvnonpar(data = cmla, epmar = TRUE, method = "p", rev = TRUE, add = TRUE, madj = 1, lty = 2)
-abvnonpar(data = cmla, epmar = TRUE, method = "p", rev = TRUE, add = TRUE, madj = 2, lty = 4)
+abvnonpar(data = cmla, epmar = TRUE, method = "pick", rev = TRUE, plot = TRUE, lty = 3)
+abvnonpar(data = cmla, epmar = TRUE, method = "pick", rev = TRUE, add = TRUE, madj = 1, lty = 2)
+abvnonpar(data = cmla, epmar = TRUE, method = "pick", rev = TRUE, add = TRUE, madj = 2, lty = 4)
 abvnonpar(data = cmla, epmar = TRUE, rev = TRUE, add = TRUE, lty = 1)
 m1 <- fbvevd(cmla, asy1 = 1, model = "alog")
 m2 <- fbvevd(cmla, model = "log")
@@ -131,7 +131,7 @@ fitted(m2); std.errors(m2)
 fitted(m3); std.errors(m3)
 
 # Figure 9.8 (a-b)
-abvnonpar(data = lossalae, k = 337, epmar = TRUE, plot = TRUE, rev = TRUE, lty = 3)
+abvnonpar(data = lossalae, method = "pot", k = 337, epmar = TRUE, plot = TRUE, rev = TRUE, lty = 3)
 plot(m1, which = 2, rev = TRUE, add = TRUE)
 plot(m2, which = 2, rev = TRUE, add = TRUE, lty = 4)
 plot(m3, which = 2, rev = TRUE, add = TRUE, lty = 2)
